using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using BEPUphysics.Entities;
using BEPUphysics;

namespace ExampleUpdateable
{
    /// <summary>
    /// Applies a force on all objects proportional to the inverse square of the distance.
    /// </summary>
    public class SphereGravity:Updateable
    {
        public Vector3 gravityCenter;

        /// <summary>
        /// Represents the gravitational constant of the field times the effective mass at the center of the field.
        /// </summary>
        public float multiplier;

        /// <summary>
        /// Represents the maximum force that can be applied by the field.
        /// </summary>
        public float maxForce;

        /// <summary>
        /// Creates a gravitational field.
        /// </summary>
        /// <param name="gravityCenter">Location of the gravity well.</param>
        /// <param name="Gm">Represents the gravitational constant of the field times the effective mass at the center of the field.</param>
        /// <param name="maxForce">Maximum force the field can apply.</param>
        public SphereGravity(Vector3 gravityCenter, float Gm, float maxForce)
        {
            this.gravityCenter = gravityCenter;
            multiplier = Gm;
            this.maxForce = maxForce;
        }

        /// <summary>
        /// Called by the space during the force application section.
        /// </summary>
        /// <param name="dt">Simulation time since last frame.</param>
        /// <param name="timeScale">Factor of time speed.</param>
        /// <param name="timeSinceLastFrame">Game time since last frame.</param>
        public override void updateDuringForces(float dt, float timeScale, float timeSinceLastFrame)
        {
            Vector3 r, impulse;
            for (int i = 0; i < space.entities.Count; i++)
            {
                r = space.entities[i].internalCenterPosition - gravityCenter;
                float distanceSquared = r.LengthSquared();
                if (distanceSquared > Toolbox.bigEpsilon) //Dividing by zero would be bad!
                {
                    float force = dt * Math.Min(maxForce, multiplier * space.entities[i].mass / distanceSquared);
                    impulse = -force * r;
                }
            }
        }

    }
}
